/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSCell;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSAttributedString;
import dwt.internal.cocoa.NSCell;
import dwt.internal.cocoa.NSColor;
import dwt.internal.cocoa.NSEvent;
import dwt.internal.cocoa.NSFont;
import dwt.internal.cocoa.NSFormatter;
import dwt.internal.cocoa.NSImage;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSMenu;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSPoint;
import dwt.internal.cocoa.NSRect;
import dwt.internal.cocoa.NSSize;
import dwt.internal.cocoa.NSText;
import dwt.internal.cocoa.NSView;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

/*alias NSUInteger NSControlSize;*/

enum NSControlSize : NSUInteger
{
    NSRegularControlSize,
    NSSmallControlSize,
    NSMiniControlSize
}

alias NSControlSize.NSRegularControlSize NSRegularControlSize;
alias NSControlSize.NSSmallControlSize NSSmallControlSize;
alias NSControlSize.NSMiniControlSize NSMiniControlSize;

/*alias NSUInteger NSCellAttribute;*/

enum NSCellAttribute : NSUInteger
{
    NSCellDisabled = 0,
    NSCellState = 1,
    NSPushInCell = 2,
    NSCellEditable = 3,
    NSChangeGrayCell = 4,
    NSCellHighlighted = 5,
    NSCellLightsByContents = 6,
    NSCellLightsByGray = 7,
    NSChangeBackgroundCell = 8,
    NSCellLightsByBackground = 9,
    NSCellIsBordered = 10,
    NSCellHasOverlappingImage = 11,
    NSCellHasImageHorizontal = 12,
    NSCellHasImageOnLeftOrBottom = 13,
    NSCellChangesContents = 14,
    NSCellIsInsetButton = 15,
    NSCellAllowsMixedState = 16
}

alias NSCellAttribute.NSCellDisabled NSCellDisabled;
alias NSCellAttribute.NSCellState NSCellState;
alias NSCellAttribute.NSPushInCell NSPushInCell;
alias NSCellAttribute.NSCellEditable NSCellEditable;
alias NSCellAttribute.NSChangeGrayCell NSChangeGrayCell;
alias NSCellAttribute.NSCellHighlighted NSCellHighlighted;
alias NSCellAttribute.NSCellLightsByContents NSCellLightsByContents;
alias NSCellAttribute.NSCellLightsByGray NSCellLightsByGray;
alias NSCellAttribute.NSChangeBackgroundCell NSChangeBackgroundCell;
alias NSCellAttribute.NSCellLightsByBackground NSCellLightsByBackground;
alias NSCellAttribute.NSCellIsBordered NSCellIsBordered;
alias NSCellAttribute.NSCellHasOverlappingImage NSCellHasOverlappingImage;
alias NSCellAttribute.NSCellHasImageHorizontal NSCellHasImageHorizontal;
alias NSCellAttribute.NSCellHasImageOnLeftOrBottom NSCellHasImageOnLeftOrBottom;
alias NSCellAttribute.NSCellChangesContents NSCellChangesContents;
alias NSCellAttribute.NSCellIsInsetButton NSCellIsInsetButton;
alias NSCellAttribute.NSCellAllowsMixedState NSCellAllowsMixedState;

enum NSControlTint : NSUInteger
{
    NSDefaultControlTint = 0,
    NSBlueControlTint = 1,
    NSGraphiteControlTint = 6,
    NSClearControlTint = 7
}

alias NSControlTint.NSDefaultControlTint NSDefaultControlTint;
alias NSControlTint.NSBlueControlTint NSBlueControlTint;
alias NSControlTint.NSGraphiteControlTint NSGraphiteControlTint;
alias NSControlTint.NSClearControlTint NSClearControlTint;

public class NSCell : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public bool acceptsFirstResponder ()
    {
        return OS.objc_msgSend(this.id, OS.sel_acceptsFirstResponder) !is null;
    }

    public objc.SEL action ()
    {
        return OS.objc_msgSend(this.id, OS.sel_action);
    }

    public objc.id alignment ()
    {
        return OS.objc_msgSend(this.id, OS.sel_alignment);
    }

    public bool allowsEditingTextAttributes ()
    {
        return OS.objc_msgSend(this.id, OS.sel_allowsEditingTextAttributes) !is null;
    }

    public bool allowsMixedState ()
    {
        return OS.objc_msgSend(this.id, OS.sel_allowsMixedState) !is null;
    }

    public bool allowsUndo ()
    {
        return OS.objc_msgSend(this.id, OS.sel_allowsUndo) !is null;
    }

    public NSAttributedString attributedStringValue ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_attributedStringValue);
        return result !is null ? new NSAttributedString(result) : null;
    }

    public objc.id backgroundStyle ()
    {
        return OS.objc_msgSend(this.id, OS.sel_backgroundStyle);
    }

    public objc.id baseWritingDirection ()
    {
        return OS.objc_msgSend(this.id, OS.sel_baseWritingDirection);
    }

    public void calcDrawInfo (NSRect aRect)
    {
        OS.objc_msgSend(this.id, OS.sel_calcDrawInfo_1, aRect);
    }

    public NSInteger cellAttribute (objc.id aParameter)
    {
        return OS.objc_msgSend(this.id, OS.sel_cellAttribute_1, aParameter);
    }

    public NSSize cellSize ()
    {
        NSSize result;
        OS.objc_msgSend_struct(result, this.id, OS.sel_cellSize);
        return result;
    }

    public NSSize cellSizeForBounds (NSRect aRect)
    {
        NSSize result;
        OS.objc_msgSend_stret(result, this.id, OS.sel_cellSizeForBounds_1, aRect);
        return result;
    }

    public objc.id compare (id otherCell)
    {
        return OS.objc_msgSend(this.id, OS.sel_compare_1, otherCell !is null ? otherCell.id : null);
    }

    public bool continueTracking (NSPoint lastPoint, NSPoint currentPoint, NSView controlView)
    {
        return OS.objc_msgSend(this.id, OS.sel_continueTracking_1at_1inView_1, lastPoint, currentPoint, controlView !is null ? controlView.id : null) !is null;
    }

    public objc.id controlSize ()
    {
        return OS.objc_msgSend(this.id, OS.sel_controlSize);
    }

    public objc.id controlTint ()
    {
        return OS.objc_msgSend(this.id, OS.sel_controlTint);
    }

    public NSView controlView ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_controlView);
        return result !is null ? new NSView(result) : null;
    }

    public static objc.id defaultFocusRingType ()
    {
        return OS.objc_msgSend(OS.class_NSCell, OS.sel_defaultFocusRingType);
    }

    public static NSMenu defaultMenu ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSCell, OS.sel_defaultMenu);
        return result !is null ? new NSMenu(result) : null;
    }

    public double doubleValue ()
    {
        return OS.objc_msgSend_fpret(this.id, OS.sel_doubleValue);
    }

    public void drawInteriorWithFrame (NSRect cellFrame, NSView controlView)
    {
        OS.objc_msgSend(this.id, OS.sel_drawInteriorWithFrame_1inView_1, cellFrame, controlView !is null ? controlView.id : null);
    }

    public void drawWithExpansionFrame (NSRect cellFrame, NSView view)
    {
        OS.objc_msgSend(this.id, OS.sel_drawWithExpansionFrame_1inView_1, cellFrame, view !is null ? view.id : null);
    }

    public void drawWithFrame (NSRect cellFrame, NSView controlView)
    {
        OS.objc_msgSend(this.id, OS.sel_drawWithFrame_1inView_1, cellFrame, controlView !is null ? controlView.id : null);
    }

    public NSRect drawingRectForBounds (NSRect theRect)
    {
        NSRect result;
        OS.objc_msgSend_stret(result, this.id, OS.sel_drawingRectForBounds_1, theRect);
        return result;
    }

    public void editWithFrame (NSRect aRect, NSView controlView, NSText textObj, id anObject, NSEvent theEvent)
    {
        OS.objc_msgSend(this.id, OS.sel_editWithFrame_1inView_1editor_1delegate_1event_1, aRect, controlView !is null ? controlView.id : null,
                textObj !is null ? textObj.id : null, anObject !is null ? anObject.id : null, theEvent !is null ? theEvent.id : null);
    }

    public void endEditing (NSText textObj)
    {
        OS.objc_msgSend(this.id, OS.sel_endEditing_1, textObj !is null ? textObj.id : null);
    }

    public NSInteger entryType ()
    {
        return OS.objc_msgSend(this.id, OS.sel_entryType);
    }

    public NSRect expansionFrameWithFrame (NSRect cellFrame, NSView view)
    {
        NSRect result;
        OS.objc_msgSend_stret(result, this.id, OS.sel_expansionFrameWithFrame_1inView_1, cellFrame, view !is null ? view.id : null);
        return result;
    }

    public float floatValue ()
    {
        return cast(float) OS.objc_msgSend_fpret(this.id, OS.sel_floatValue);
    }

    public objc.id focusRingType ()
    {
        return OS.objc_msgSend(this.id, OS.sel_focusRingType);
    }

    public NSFont font ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_font);
        return result !is null ? new NSFont(result) : null;
    }

    public id formatter ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_formatter);
        return result !is null ? new id(result) : null;
    }

    public void getPeriodicDelay (float* delay, float* interval)
    {
        OS.objc_msgSend(this.id, OS.sel_getPeriodicDelay_1interval_1, delay, interval);
    }

    public bool hasValidObjectValue ()
    {
        return OS.objc_msgSend(this.id, OS.sel_hasValidObjectValue) !is null;
    }

    public void highlight (bool flag, NSRect cellFrame, NSView controlView)
    {
        OS.objc_msgSend(this.id, OS.sel_highlight_1withFrame_1inView_1, flag, cellFrame, controlView !is null ? controlView.id : null);
    }

    public NSColor highlightColorWithFrame (NSRect cellFrame, NSView controlView)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_highlightColorWithFrame_1inView_1, cellFrame, controlView !is null ? controlView.id : null);
        return result !is null ? new NSColor(result) : null;
    }

    public NSUInteger hitTestForEvent (NSEvent event, NSRect cellFrame, NSView controlView)
    {
        return OS.objc_msgSend(this.id, OS.sel_hitTestForEvent_1inRect_1ofView_1, event !is null ? event.id : null, cellFrame,
                controlView !is null ? controlView.id : null);
    }

    public NSImage image ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_image);
        return result !is null ? new NSImage(result) : null;
    }

    public NSRect imageRectForBounds (NSRect theRect)
    {
        NSRect result;
        OS.objc_msgSend_stret(result, this.id, OS.sel_imageRectForBounds_1, theRect);
        return result;
    }

    public bool importsGraphics ()
    {
        return OS.objc_msgSend(this.id, OS.sel_importsGraphics) !is null;
    }

    public NSCell initImageCell (NSImage image)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initImageCell_1, image !is null ? image.id : null);
        return result !is null ? this : null;
    }

    public NSCell initTextCell (NSString aString)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initTextCell_1, aString !is null ? aString.id : null);
        return result !is null ? this : null;
    }

    public int intValue ()
    {
        return OS.objc_msgSend(this.id, OS.sel_intValue);
    }

    public int integerValue ()
    {
        return OS.objc_msgSend(this.id, OS.sel_integerValue);
    }

    public objc.id interiorBackgroundStyle ()
    {
        return OS.objc_msgSend(this.id, OS.sel_interiorBackgroundStyle);
    }

    public bool isBezeled ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isBezeled) !is null;
    }

    public bool isBordered ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isBordered) !is null;
    }

    public bool isContinuous ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isContinuous) !is null;
    }

    public bool isEditable ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isEditable) !is null;
    }

    public bool isEnabled ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isEnabled) !is null;
    }

    public bool isEntryAcceptable (NSString aString)
    {
        return OS.objc_msgSend(this.id, OS.sel_isEntryAcceptable_1, aString !is null ? aString.id : null) !is null;
    }

    public bool isHighlighted ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isHighlighted) !is null;
    }

    public bool isOpaque ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isOpaque) !is null;
    }

    public bool isScrollable ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isScrollable) !is null;
    }

    public bool isSelectable ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isSelectable) !is null;
    }

    public NSString keyEquivalent ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_keyEquivalent);
        return result !is null ? new NSString(result) : null;
    }

    public objc.id lineBreakMode ()
    {
        return OS.objc_msgSend(this.id, OS.sel_lineBreakMode);
    }

    public NSMenu menu ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_menu);
        return result !is null ? new NSMenu(result) : null;
    }

    public NSMenu menuForEvent (NSEvent event, NSRect cellFrame, NSView view)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_menuForEvent_1inRect_1ofView_1, event !is null ? event.id : null, cellFrame,
                view !is null ? view.id : null);
        return result !is null ? new NSMenu(result) : null;
    }

    public NSString mnemonic ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_mnemonic);
        return result !is null ? new NSString(result) : null;
    }

    public NSUInteger mnemonicLocation ()
    {
        return OS.objc_msgSend(this.id, OS.sel_mnemonicLocation);
    }

    public NSInteger mouseDownFlags ()
    {
        return OS.objc_msgSend(this.id, OS.sel_mouseDownFlags);
    }

    public NSInteger nextState ()
    {
        return OS.objc_msgSend(this.id, OS.sel_nextState);
    }

    public id objectValue ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_objectValue);
        return result !is null ? new id(result) : null;
    }

    public void performClick (id sender)
    {
        OS.objc_msgSend(this.id, OS.sel_performClick_1, sender !is null ? sender.id : null);
    }

    public static bool prefersTrackingUntilMouseUp ()
    {
        return OS.objc_msgSend(OS.class_NSCell, OS.sel_prefersTrackingUntilMouseUp) !is null;
    }

    public bool refusesFirstResponder ()
    {
        return OS.objc_msgSend(this.id, OS.sel_refusesFirstResponder) !is null;
    }

    public id representedObject ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_representedObject);
        return result !is null ? new id(result) : null;
    }

    public void resetCursorRect (NSRect cellFrame, NSView controlView)
    {
        OS.objc_msgSend(this.id, OS.sel_resetCursorRect_1inView_1, cellFrame, controlView !is null ? controlView.id : null);
    }

    public void selectWithFrame (NSRect aRect, NSView controlView, NSText textObj, id anObject, NSInteger selStart, NSInteger selLength)
    {
        OS.objc_msgSend(this.id, OS.sel_selectWithFrame_1inView_1editor_1delegate_1start_1length_1, aRect,
                controlView !is null ? controlView.id : null, textObj !is null ? textObj.id : null, anObject !is null ? anObject.id : null, selStart,
                selLength);
    }

    public NSInteger sendActionOn (NSInteger mask)
    {
        return OS.objc_msgSend(this.id, OS.sel_sendActionOn_1, mask);
    }

    public bool sendsActionOnEndEditing ()
    {
        return OS.objc_msgSend(this.id, OS.sel_sendsActionOnEndEditing) !is null;
    }

    public void setAction (objc.SEL aSelector)
    {
        OS.objc_msgSend(this.id, OS.sel_setAction_1, aSelector);
    }

    public void setAlignment (objc.id mode)
    {
        OS.objc_msgSend(this.id, OS.sel_setAlignment_1, mode);
    }

    public void setAllowsEditingTextAttributes (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setAllowsEditingTextAttributes_1, flag);
    }

    public void setAllowsMixedState (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setAllowsMixedState_1, flag);
    }

    public void setAllowsUndo (bool allowsUndo)
    {
        OS.objc_msgSend(this.id, OS.sel_setAllowsUndo_1, allowsUndo);
    }

    public void setAttributedStringValue (NSAttributedString obj)
    {
        OS.objc_msgSend(this.id, OS.sel_setAttributedStringValue_1, obj !is null ? obj.id : null);
    }

    public void setBackgroundStyle (objc.id style)
    {
        OS.objc_msgSend(this.id, OS.sel_setBackgroundStyle_1, style);
    }

    public void setBaseWritingDirection (objc.id writingDirection)
    {
        OS.objc_msgSend(this.id, OS.sel_setBaseWritingDirection_1, writingDirection);
    }

    public void setBezeled (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setBezeled_1, flag);
    }

    public void setBordered (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setBordered_1, flag);
    }

    public void setCellAttribute (NSCellAttribute aParameter, NSInteger value)
    {
        OS.objc_msgSend(this.id, OS.sel_setCellAttribute_1to_1, aParameter, value);
    }

    public void setContinuous (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setContinuous_1, flag);
    }

    public void setControlSize (objc.id size)
    {
        OS.objc_msgSend(this.id, OS.sel_setControlSize_1, size);
    }

    public void setControlTint (objc.id controlTint)
    {
        OS.objc_msgSend(this.id, OS.sel_setControlTint_1, controlTint);
    }

    public void setControlView (NSView view)
    {
        OS.objc_msgSend(this.id, OS.sel_setControlView_1, view !is null ? view.id : null);
    }

    public void setDoubleValue (double aDouble)
    {
        OS.objc_msgSend(this.id, OS.sel_setDoubleValue_1, aDouble);
    }

    public void setEditable (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setEditable_1, flag);
    }

    public void setEnabled (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setEnabled_1, flag);
    }

    public void setEntryType (NSInteger aType)
    {
        OS.objc_msgSend(this.id, OS.sel_setEntryType_1, aType);
    }

    public void setFloatValue (float aFloat)
    {
        OS.objc_msgSend(this.id, OS.sel_setFloatValue_1, aFloat);
    }

    public void setFloatingPointFormat (bool autoRange, NSUInteger leftDigits, NSUInteger rightDigits)
    {
        OS.objc_msgSend(this.id, OS.sel_setFloatingPointFormat_1left_1right_1, autoRange, leftDigits, rightDigits);
    }

    public void setFocusRingType (objc.id focusRingType)
    {
        OS.objc_msgSend(this.id, OS.sel_setFocusRingType_1, focusRingType);
    }

    public void setFont (NSFont fontObj)
    {
        OS.objc_msgSend(this.id, OS.sel_setFont_1, fontObj !is null ? fontObj.id : null);
    }

    public void setFormatter (NSFormatter newFormatter)
    {
        OS.objc_msgSend(this.id, OS.sel_setFormatter_1, newFormatter !is null ? newFormatter.id : null);
    }

    public void setHighlighted (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setHighlighted_1, flag);
    }

    public void setImage (NSImage image)
    {
        OS.objc_msgSend(this.id, OS.sel_setImage_1, image !is null ? image.id : null);
    }

    public void setImportsGraphics (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setImportsGraphics_1, flag);
    }

    public void setIntValue (int anInt)
    {
        OS.objc_msgSend(this.id, OS.sel_setIntValue_1, anInt);
    }

    public void setIntegerValue (int anInteger)
    {
        OS.objc_msgSend(this.id, OS.sel_setIntegerValue_1, anInteger);
    }

    public void setLineBreakMode (objc.id mode)
    {
        OS.objc_msgSend(this.id, OS.sel_setLineBreakMode_1, mode);
    }

    public void setMenu (NSMenu aMenu)
    {
        OS.objc_msgSend(this.id, OS.sel_setMenu_1, aMenu !is null ? aMenu.id : null);
    }

    public void setMnemonicLocation (NSUInteger location)
    {
        OS.objc_msgSend(this.id, OS.sel_setMnemonicLocation_1, location);
    }

    public void setNextState ()
    {
        OS.objc_msgSend(this.id, OS.sel_setNextState);
    }

    public void setObjectValue (id obj)
    {
        OS.objc_msgSend(this.id, OS.sel_setObjectValue_1, obj !is null ? obj.id : null);
    }

    public void setRefusesFirstResponder (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setRefusesFirstResponder_1, flag);
    }

    public void setRepresentedObject (id anObject)
    {
        OS.objc_msgSend(this.id, OS.sel_setRepresentedObject_1, anObject !is null ? anObject.id : null);
    }

    public void setScrollable (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setScrollable_1, flag);
    }

    public void setSelectable (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setSelectable_1, flag);
    }

    public void setSendsActionOnEndEditing (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setSendsActionOnEndEditing_1, flag);
    }

    public void setShowsFirstResponder (bool showFR)
    {
        OS.objc_msgSend(this.id, OS.sel_setShowsFirstResponder_1, showFR);
    }

    public void setState (NSInteger value)
    {
        OS.objc_msgSend(this.id, OS.sel_setState_1, value);
    }

    public void setStringValue (NSString aString)
    {
        OS.objc_msgSend(this.id, OS.sel_setStringValue_1, aString !is null ? aString.id : null);
    }

    public void setTag (NSInteger anInt)
    {
        OS.objc_msgSend(this.id, OS.sel_setTag_1, anInt);
    }

    public void setTarget (id anObject)
    {
        OS.objc_msgSend(this.id, OS.sel_setTarget_1, anObject !is null ? anObject.id : null);
    }

    public void setTitle (NSString aString)
    {
        OS.objc_msgSend(this.id, OS.sel_setTitle_1, aString !is null ? aString.id : null);
    }

    public void setTitleWithMnemonic (NSString StringWithAmpersand)
    {
        OS.objc_msgSend(this.id, OS.sel_setTitleWithMnemonic_1, StringWithAmpersand !is null ? StringWithAmpersand.id : null);
    }

    public void setTruncatesLastVisibleLine (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setTruncatesLastVisibleLine_1, flag);
    }

    public void setType (objc.id aType)
    {
        OS.objc_msgSend(this.id, OS.sel_setType_1, aType);
    }

    public NSText setUpFieldEditorAttributes (NSText textObj)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_setUpFieldEditorAttributes_1, textObj !is null ? textObj.id : null);
        return result !is null ? new NSText(result) : null;
    }

    public void setWraps (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setWraps_1, flag);
    }

    public bool showsFirstResponder ()
    {
        return OS.objc_msgSend(this.id, OS.sel_showsFirstResponder) !is null;
    }

    public bool startTrackingAt (NSPoint startPoint, NSView controlView)
    {
        return OS.objc_msgSend(this.id, OS.sel_startTrackingAt_1inView_1, startPoint, controlView !is null ? controlView.id : null) !is null;
    }

    public NSInteger state ()
    {
        return OS.objc_msgSend(this.id, OS.sel_state);
    }

    public void stopTracking (NSPoint lastPoint, NSPoint stopPoint, NSView controlView, bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_stopTracking_1at_1inView_1mouseIsUp_1, lastPoint, stopPoint, controlView !is null ? controlView.id : null,
                flag);
    }

    public NSString StringValue ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_StringValue);
        return result !is null ? new NSString(result) : null;
    }

    public NSInteger tag ()
    {
        return OS.objc_msgSend(this.id, OS.sel_tag);
    }

    public void takeDoubleValueFrom (id sender)
    {
        OS.objc_msgSend(this.id, OS.sel_takeDoubleValueFrom_1, sender !is null ? sender.id : null);
    }

    public void takeFloatValueFrom (id sender)
    {
        OS.objc_msgSend(this.id, OS.sel_takeFloatValueFrom_1, sender !is null ? sender.id : null);
    }

    public void takeIntValueFrom (id sender)
    {
        OS.objc_msgSend(this.id, OS.sel_takeIntValueFrom_1, sender !is null ? sender.id : null);
    }

    public void takeIntegerValueFrom (id sender)
    {
        OS.objc_msgSend(this.id, OS.sel_takeIntegerValueFrom_1, sender !is null ? sender.id : null);
    }

    public void takeObjectValueFrom (id sender)
    {
        OS.objc_msgSend(this.id, OS.sel_takeObjectValueFrom_1, sender !is null ? sender.id : null);
    }

    public void takeStringValueFrom (id sender)
    {
        OS.objc_msgSend(this.id, OS.sel_takeStringValueFrom_1, sender !is null ? sender.id : null);
    }

    public id target ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_target);
        return result !is null ? new id(result) : null;
    }

    public NSString title ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_title);
        return result !is null ? new NSString(result) : null;
    }

    public NSRect titleRectForBounds (NSRect theRect)
    {
        NSRect result;
        OS.objc_msgSend_stret(result, this.id, OS.sel_titleRectForBounds_1, theRect);
        return result;
    }

    public bool trackMouse (NSEvent theEvent, NSRect cellFrame, NSView controlView, bool flag)
    {
        return OS.objc_msgSend(this.id, OS.sel_trackMouse_1inRect_1ofView_1untilMouseUp_1, theEvent !is null ? theEvent.id : null, cellFrame,
                controlView !is null ? controlView.id : null, flag) !is null;
    }

    public bool truncatesLastVisibleLine ()
    {
        return OS.objc_msgSend(this.id, OS.sel_truncatesLastVisibleLine) !is null;
    }

    public objc.id type ()
    {
        return OS.objc_msgSend(this.id, OS.sel_type);
    }

    public bool wantsNotificationForMarkedText ()
    {
        return OS.objc_msgSend(this.id, OS.sel_wantsNotificationForMarkedText) !is null;
    }

    public bool wraps ()
    {
        return OS.objc_msgSend(this.id, OS.sel_wraps) !is null;
    }

}
