/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSForm;

import dwt.internal.cocoa.CGFloat;
import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSFont;
import dwt.internal.cocoa.NSFormCell;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSMatrix;
import dwt.internal.cocoa.NSSize;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.NSText : NSTextAlignment, NSWritingDirection;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSForm : NSMatrix
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSFormCell addEntry (NSString title)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_addEntry_1, title !is null ? title.id : null);
        return result !is null ? new NSFormCell(result) : null;
    }

    public id cellAtIndex (NSInteger index)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_cellAtIndex_1, index);
        return result !is null ? new id(result) : null;
    }

    public void drawCellAtIndex (NSInteger index)
    {
        OS.objc_msgSend(this.id, OS.sel_drawCellAtIndex_1, index);
    }

    public NSInteger indexOfCellWithTag (NSInteger aTag)
    {
        return OS.objc_msgSend(this.id, OS.sel_indexOfCellWithTag_1, aTag);
    }

    public NSInteger indexOfSelectedItem ()
    {
        return OS.objc_msgSend(this.id, OS.sel_indexOfSelectedItem);
    }

    public NSFormCell insertEntry (NSString title, int index)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_insertEntry_1atIndex_1, title !is null ? title.id : null, index);
        return result !is null ? new NSFormCell(result) : null;
    }

    public void removeEntryAtIndex (NSInteger index)
    {
        OS.objc_msgSend(this.id, OS.sel_removeEntryAtIndex_1, index);
    }

    public void selectTextAtIndex (NSInteger index)
    {
        OS.objc_msgSend(this.id, OS.sel_selectTextAtIndex_1, index);
    }

    public void setBezeled (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setBezeled_1, flag);
    }

    public void setBordered (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setBordered_1, flag);
    }

    public void setEntryWidth (CGFloat width)
    {
        OS.objc_msgSend(this.id, OS.sel_setEntryWidth_1, width);
    }

    public void setFrameSize (NSSize newSize)
    {
        OS.objc_msgSend(this.id, OS.sel_setFrameSize_1, newSize);
    }

    public void setInterlineSpacing (CGFloat spacing)
    {
        OS.objc_msgSend(this.id, OS.sel_setInterlineSpacing_1, spacing);
    }

    public void setTextAlignment (NSInteger mode)
    {
        OS.objc_msgSend(this.id, OS.sel_setTextAlignment_1, mode);
    }

    public void setTextBaseWritingDirection (NSWritingDirection writingDirection)
    {
        OS.objc_msgSend(this.id, OS.sel_setTextBaseWritingDirection_1, writingDirection);
    }

    public void setTextFont (NSFont fontObj)
    {
        OS.objc_msgSend(this.id, OS.sel_setTextFont_1, fontObj !is null ? fontObj.id : null);
    }

    public void setTitleAlignment (NSTextAlignment mode)
    {
        OS.objc_msgSend(this.id, OS.sel_setTitleAlignment_1, mode);
    }

    public void setTitleBaseWritingDirection (NSWritingDirection writingDirection)
    {
        OS.objc_msgSend(this.id, OS.sel_setTitleBaseWritingDirection_1, writingDirection);
    }

    public void setTitleFont (NSFont fontObj)
    {
        OS.objc_msgSend(this.id, OS.sel_setTitleFont_1, fontObj !is null ? fontObj.id : null);
    }

}
