/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSHTTPCookieStorage;

import dwt.internal.cocoa.NSArray;
import dwt.internal.cocoa.NSHTTPCookie;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSURL;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

enum NSHTTPCookieAcceptPolicy
{
    NSHTTPCookieAcceptPolicyAlways,
    NSHTTPCookieAcceptPolicyNever,
    NSHTTPCookieAcceptPolicyOnlyFromMainDocumentDomain
}

public class NSHTTPCookieStorage : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSHTTPCookieAcceptPolicy cookieAcceptPolicy ()
    {
        return OS.objc_msgSend(this.id, OS.sel_cookieAcceptPolicy);
    }

    public NSArray cookies ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_cookies);
        return result !is null ? new NSArray(result) : null;
    }

    public NSArray cookiesForURL (NSURL URL)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_cookiesForURL_1, URL !is null ? URL.id : null);
        return result !is null ? new NSArray(result) : null;
    }

    public void deleteCookie (NSHTTPCookie cookie)
    {
        OS.objc_msgSend(this.id, OS.sel_deleteCookie_1, cookie !is null ? cookie.id : null);
    }

    public void setCookie (NSHTTPCookie cookie)
    {
        OS.objc_msgSend(this.id, OS.sel_setCookie_1, cookie !is null ? cookie.id : null);
    }

    public void setCookieAcceptPolicy (NSHTTPCookieAcceptPolicy cookieAcceptPolicy)
    {
        OS.objc_msgSend(this.id, OS.sel_setCookieAcceptPolicy_1, cookieAcceptPolicy);
    }

    public void setCookies (NSArray cookies, NSURL URL, NSURL mainDocumentURL)
    {
        OS.objc_msgSend(this.id, OS.sel_setCookies_1forURL_1mainDocumentURL_1, cookies !is null ? cookies.id : null, URL !is null ? URL.id : null,
                mainDocumentURL !is null ? mainDocumentURL.id : null);
    }

    public static NSHTTPCookieStorage sharedHTTPCookieStorage ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSHTTPCookieStorage, OS.sel_sharedHTTPCookieStorage);
        return result !is null ? new NSHTTPCookieStorage(result) : null;
    }

}
