/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSScreen;

import dwt.internal.cocoa.CGFloat;
import dwt.internal.cocoa.NSArray;
import dwt.internal.cocoa.NSDictionary;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSRect;
import dwt.internal.cocoa.NSWindow : NSWindowDepth;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSScreen : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public static NSScreen deepestScreen ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSScreen, OS.sel_deepestScreen);
        return result !is null ? new NSScreen(result) : null;
    }

    public NSWindowDepth depth ()
    {
        return OS.objc_msgSend(this.id, OS.sel_depth);
    }

    public NSDictionary deviceDescription ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_deviceDescription);
        return result !is null ? new NSDictionary(result) : null;
    }

    public NSRect frame ()
    {
        NSRect result;
        OS.objc_msgSend_stret(result, this.id, OS.sel_frame);
        return result;
    }

    public static NSScreen mainScreen ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSScreen, OS.sel_mainScreen);
        return result !is null ? new NSScreen(result) : null;
    }

    public static NSArray screens ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSScreen, OS.sel_screens);
        return result !is null ? new NSArray(result) : null;
    }

    public /*const*/NSWindowDepth* supportedWindowDepths ()
    {
        return cast(/*const*/NSWindowDepth*)OS.objc_msgSend(this.id, OS.sel_supportedWindowDepths);
    }

    public CGFloat userSpaceScaleFactor ()
    {
        return cast(CGFloat) OS.objc_msgSend_fpret(this.id, OS.sel_userSpaceScaleFactor);
    }

    public NSRect visibleFrame ()
    {
        NSRect result;
        OS.objc_msgSend_stret(result, this.id, OS.sel_visibleFrame);
        return result;
    }

}
