/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSScriptSuiteRegistry;

import dwt.internal.c.carboncore.MacTypes : FourCharCodes;
import dwt.internal.cocoa.NSArray;
import dwt.internal.cocoa.NSBundle;
import dwt.internal.cocoa.NSData;
import dwt.internal.cocoa.NSDictionary;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSScriptClassDescription;
import dwt.internal.cocoa.NSScriptCommandDescription;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSScriptSuiteRegistry : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSData aeteResource (NSString languageName)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_aeteResource_1, languageName !is null ? languageName.id : null);
        return result !is null ? new NSData(result) : null;
    }

    public FourCharCode appleEventCodeForSuite (NSString suiteName)
    {
        return cast(FourCharCode) OS.objc_msgSend(this.id, OS.sel_appleEventCodeForSuite_1, suiteName !is null ? suiteName.id : null);
    }

    public NSBundle bundleForSuite (NSString suiteName)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_bundleForSuite_1, suiteName !is null ? suiteName.id : null);
        return result !is null ? new NSBundle(result) : null;
    }

    public NSScriptClassDescription classDescriptionWithAppleEventCode (FourCharCode appleEventCode)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_classDescriptionWithAppleEventCode_1, appleEventCode);
        return result !is null ? new NSScriptClassDescription(result) : null;
    }

    public NSDictionary classDescriptionsInSuite (NSString suiteName)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_classDescriptionsInSuite_1, suiteName !is null ? suiteName.id : null);
        return result !is null ? new NSDictionary(result) : null;
    }

    public NSScriptCommandDescription commandDescriptionWithAppleEventClass (FourCharCode appleEventClassCode, FourCharCode appleEventIDCode)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_commandDescriptionWithAppleEventClass_1andAppleEventCode_1, appleEventClassCode,
                appleEventIDCode);
        return result !is null ? new NSScriptCommandDescription(result) : null;
    }

    public NSDictionary commandDescriptionsInSuite (NSString suiteName)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_commandDescriptionsInSuite_1, suiteName !is null ? suiteName.id : null);
        return result !is null ? new NSDictionary(result) : null;
    }

    public void loadSuiteWithDictionary (NSDictionary suiteDeclaration, NSBundle bundle)
    {
        OS.objc_msgSend(this.id, OS.sel_loadSuiteWithDictionary_1fromBundle_1, suiteDeclaration !is null ? suiteDeclaration.id : null,
                bundle !is null ? bundle.id : null);
    }

    public void loadSuitesFromBundle (NSBundle bundle)
    {
        OS.objc_msgSend(this.id, OS.sel_loadSuitesFromBundle_1, bundle !is null ? bundle.id : null);
    }

    public void registerClassDescription (NSScriptClassDescription classDescription)
    {
        OS.objc_msgSend(this.id, OS.sel_registerClassDescription_1, classDescription !is null ? classDescription.id : null);
    }

    public void registerCommandDescription (NSScriptCommandDescription commandDescription)
    {
        OS.objc_msgSend(this.id, OS.sel_registerCommandDescription_1, commandDescription !is null ? commandDescription.id : null);
    }

    public static void setSharedScriptSuiteRegistry (NSScriptSuiteRegistry registry)
    {
        OS.objc_msgSend(OS.class_NSScriptSuiteRegistry, OS.sel_setSharedScriptSuiteRegistry_1, registry !is null ? registry.id : null);
    }

    public static NSScriptSuiteRegistry sharedScriptSuiteRegistry ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSScriptSuiteRegistry, OS.sel_sharedScriptSuiteRegistry);
        return result !is null ? new NSScriptSuiteRegistry(result) : null;
    }

    public NSString suiteForAppleEventCode (FourCharCode appleEventCode)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_suiteForAppleEventCode_1, appleEventCode);
        return result !is null ? new NSString(result) : null;
    }

    public NSArray suiteNames ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_suiteNames);
        return result !is null ? new NSArray(result) : null;
    }

}
