/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSSocketPortNameServer;

import dwt.internal.cocoa.NSPort;
import dwt.internal.cocoa.NSPortNameServer;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSSocketPortNameServer : NSPortNameServer
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public ushort defaultNameServerPortNumber ()
    {
        return cast(ushort) OS.objc_msgSend(this.id, OS.sel_defaultNameServerPortNumber);
    }

    public NSPort portForName_ (NSString name)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_portForName_1, name !is null ? name.id : null);
        return result !is null ? new NSPort(result) : null;
    }

    public NSPort portForName_host_ (NSString name, NSString host)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_portForName_1host_1, name !is null ? name.id : null, host !is null ? host.id : null);
        return result !is null ? new NSPort(result) : null;
    }

    public NSPort portForName_host_nameServerPortNumber_ (NSString name, NSString host, ushort portNumber)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_portForName_1host_1nameServerPortNumber_1, name !is null ? name.id : null,
                host !is null ? host.id : null, portNumber);
        return result !is null ? new NSPort(result) : null;
    }

    public bool registerPort_name_ (NSPort port, NSString name)
    {
        return OS.objc_msgSend(this.id, OS.sel_registerPort_1name_1, port !is null ? port.id : null, name !is null ? name.id : null) !is null;
    }

    public bool registerPort_name_nameServerPortNumber_ (NSPort port, NSString name, ushort portNumber)
    {
        return OS.objc_msgSend(this.id, OS.sel_registerPort_1name_1nameServerPortNumber_1, port !is null ? port.id : null,
                name !is null ? name.id : null, portNumber) !is null;
    }

    public bool removePortForName (NSString name)
    {
        return OS.objc_msgSend(this.id, OS.sel_removePortForName_1, name !is null ? name.id : null) !is null;
    }

    public void setDefaultNameServerPortNumber (ushort portNumber)
    {
        OS.objc_msgSend(this.id, OS.sel_setDefaultNameServerPortNumber_1, portNumber);
    }

    public static id sharedInstance ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSSocketPortNameServer, OS.sel_sharedInstance);
        return result !is null ? new id(result) : null;
    }

}
