/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSTableColumn;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSCell;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSSortDescriptor;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.NSTableHeaderCell;
import dwt.internal.cocoa.NSTableView;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSTableColumn : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public id dataCell ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_dataCell);
        return result !is null ? new id(result) : null;
    }

    public id dataCellForRow (NSInteger row)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_dataCellForRow_1, row);
        return result !is null ? new id(result) : null;
    }

    public NSTableHeaderCell headerCell ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_headerCell);
        return result !is null ? new NSTableHeaderCell(result) : null;
    }

    public NSString headerToolTip ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_headerToolTip);
        return result !is null ? new NSString(result) : null;
    }

    public id identifier ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_identifier);
        return result !is null ? new id(result) : null;
    }

    public NSTableColumn initWithIdentifier (id identifier)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithIdentifier_1, identifier !is null ? identifier.id : null);
        return result !is null ? this : null;
    }

    public bool isEditable ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isEditable) !is null;
    }

    public bool isHidden ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isHidden) !is null;
    }

    public bool isResizable ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isResizable) !is null;
    }

    public CGFloat maxWidth ()
    {
        return cast(CGFloat) OS.objc_msgSend_fpret(this.id, OS.sel_maxWidth);
    }

    public CGFloat minWidth ()
    {
        return cast(CGFloat) OS.objc_msgSend_fpret(this.id, OS.sel_minWidth);
    }

    public NSUInteger resizingMask ()
    {
        return cast(NSUInteger) OS.objc_msgSend(this.id, OS.sel_resizingMask);
    }

    public void setDataCell (NSCell cell)
    {
        OS.objc_msgSend(this.id, OS.sel_setDataCell_1, cell !is null ? cell.id : null);
    }

    public void setEditable (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setEditable_1, flag);
    }

    public void setHeaderCell (NSCell cell)
    {
        OS.objc_msgSend(this.id, OS.sel_setHeaderCell_1, cell !is null ? cell.id : null);
    }

    public void setHeaderToolTip (NSString string)
    {
        OS.objc_msgSend(this.id, OS.sel_setHeaderToolTip_1, string !is null ? string.id : null);
    }

    public void setHidden (bool hidden)
    {
        OS.objc_msgSend(this.id, OS.sel_setHidden_1, hidden);
    }

    public void setIdentifier (id identifier)
    {
        OS.objc_msgSend(this.id, OS.sel_setIdentifier_1, identifier !is null ? identifier.id : null);
    }

    public void setMaxWidth (CGFloat maxWidth)
    {
        OS.objc_msgSend(this.id, OS.sel_setMaxWidth_1, maxWidth);
    }

    public void setMinWidth (CGFloat minWidth)
    {
        OS.objc_msgSend(this.id, OS.sel_setMinWidth_1, minWidth);
    }

    public void setResizable (bool flag)
    {
        OS.objc_msgSend(this.id, OS.sel_setResizable_1, flag);
    }

    public void setResizingMask (NSUInteger resizingMask)
    {
        OS.objc_msgSend(this.id, OS.sel_setResizingMask_1, resizingMask);
    }

    public void setSortDescriptorPrototype (NSSortDescriptor sortDescriptor)
    {
        OS.objc_msgSend(this.id, OS.sel_setSortDescriptorPrototype_1, sortDescriptor !is null ? sortDescriptor.id : null);
    }

    public void setTableView (NSTableView tableView)
    {
        OS.objc_msgSend(this.id, OS.sel_setTableView_1, tableView !is null ? tableView.id : null);
    }

    public void setWidth (CGFloat width)
    {
        OS.objc_msgSend(this.id, OS.sel_setWidth_1, width);
    }

    public void sizeToFit ()
    {
        OS.objc_msgSend(this.id, OS.sel_sizeToFit);
    }

    public NSSortDescriptor sortDescriptorPrototype ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_sortDescriptorPrototype);
        return result !is null ? new NSSortDescriptor(result) : null;
    }

    public NSTableView tableView ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_tableView);
        return result !is null ? new NSTableView(result) : null;
    }

    public CGFloat width ()
    {
        return cast(CGFloat) OS.objc_msgSend_fpret(this.id, OS.sel_width);
    }

}
