/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSXMLDTDNode;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.NSXMLNode;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;



enum NSXMLDTDNodeKind : NSUInteger
{
    NSXMLEntityGeneralKind = 1,
    NSXMLEntityParsedKind,
    NSXMLEntityUnparsedKind,
    NSXMLEntityParameterKind,
    NSXMLEntityPredefined,

    NSXMLAttributeCDATAKind,
    NSXMLAttributeIDKind,
    NSXMLAttributeIDRefKind,
    NSXMLAttributeIDRefsKind,
    NSXMLAttributeEntityKind,
    NSXMLAttributeEntitiesKind,
    NSXMLAttributeNMTokenKind,
    NSXMLAttributeNMTokensKind,
    NSXMLAttributeEnumerationKind,
    NSXMLAttributeNotationKind,

    NSXMLElementDeclarationUndefinedKind,
    NSXMLElementDeclarationEmptyKind,
    NSXMLElementDeclarationAnyKind,
    NSXMLElementDeclarationMixedKind,
    NSXMLElementDeclarationElementKind
}

alias NSXMLDTDNodeKind.NSXMLEntityGeneralKind NSXMLEntityGeneralKind;
alias NSXMLDTDNodeKind.NSXMLEntityParsedKind NSXMLEntityParsedKind;
alias NSXMLDTDNodeKind.NSXMLEntityUnparsedKind NSXMLEntityUnparsedKind;
alias NSXMLDTDNodeKind.NSXMLEntityParameterKind NSXMLEntityParameterKind;
alias NSXMLDTDNodeKind.NSXMLEntityPredefined NSXMLEntityPredefined;

alias NSXMLDTDNodeKind.NSXMLAttributeCDATAKind NSXMLAttributeCDATAKind;
alias NSXMLDTDNodeKind.NSXMLAttributeIDKind NSXMLAttributeIDKind;
alias NSXMLDTDNodeKind.NSXMLAttributeIDRefKind NSXMLAttributeIDRefKind;
alias NSXMLDTDNodeKind.NSXMLAttributeIDRefsKind NSXMLAttributeIDRefsKind;
alias NSXMLDTDNodeKind.NSXMLAttributeEntityKind NSXMLAttributeEntityKind;
alias NSXMLDTDNodeKind.NSXMLAttributeEntitiesKind NSXMLAttributeEntitiesKind;
alias NSXMLDTDNodeKind.NSXMLAttributeNMTokenKind NSXMLAttributeNMTokenKind;
alias NSXMLDTDNodeKind.NSXMLAttributeNMTokensKind NSXMLAttributeNMTokensKind;
alias NSXMLDTDNodeKind.NSXMLAttributeEnumerationKind NSXMLAttributeEnumerationKind;
alias NSXMLDTDNodeKind.NSXMLAttributeNotationKind NSXMLAttributeNotationKind;

alias NSXMLDTDNodeKind.NSXMLElementDeclarationUndefinedKind NSXMLElementDeclarationUndefinedKind;
alias NSXMLDTDNodeKind.NSXMLElementDeclarationEmptyKind NSXMLElementDeclarationEmptyKind;
alias NSXMLDTDNodeKind.NSXMLElementDeclarationAnyKind NSXMLElementDeclarationAnyKind;
alias NSXMLDTDNodeKind.NSXMLElementDeclarationMixedKind NSXMLElementDeclarationMixedKind;
alias NSXMLDTDNodeKind.NSXMLElementDeclarationElementKind NSXMLElementDeclarationElementKind;



public class NSXMLDTDNode : NSXMLNode
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSXMLDTDNodeKind DTDKind ()
    {
        return cast(NSXMLDTDNodeKind) OS.objc_msgSend(this.id, OS.sel_DTDKind);
    }

    public id initWithXMLString (NSString string)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithXMLString_1, string !is null ? string.id : null);
        return result !is null ? new id(result) : null;
    }

    public bool isExternal ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isExternal) !is null;
    }

    public NSString notationName ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_notationName);
        return result !is null ? new NSString(result) : null;
    }

    public NSString publicID ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_publicID);
        return result !is null ? new NSString(result) : null;
    }

    public void setDTDKind (NSXMLDTDNodeKind kind)
    {
        OS.objc_msgSend(this.id, OS.sel_setDTDKind_1, kind);
    }

    public void setNotationName (NSString notationName)
    {
        OS.objc_msgSend(this.id, OS.sel_setNotationName_1, notationName !is null ? notationName.id : null);
    }

    public void setPublicID (NSString publicID)
    {
        OS.objc_msgSend(this.id, OS.sel_setPublicID_1, publicID !is null ? publicID.id : null);
    }

    public void setSystemID (NSString systemID)
    {
        OS.objc_msgSend(this.id, OS.sel_setSystemID_1, systemID !is null ? systemID.id : null);
    }

    public NSString systemID ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_systemID);
        return result !is null ? new NSString(result) : null;
    }

}
